@@ -46,6 +46,8 @@ mail_item_free (MailItem *item)
 	exchange_mapi_util_free_stream_list (&item->generic_streams);
 	exchange_mapi_util_free_recipient_list (&item->recipients);
 
+	g_free (item->msg_class);
+
 	g_free (item);
 }
 
@@ -99,6 +101,8 @@ fetch_props_to_mail_item_cb (FetchItemsCallbackData *item_data, gpointer data)
 		}
 	}
 
+	item->msg_class = g_strdup (msg_class);
+
 	item->is_cal = FALSE;
 	if (msg_class && g_str_has_prefix (msg_class, IPM_SCHEDULE_MEETING_PREFIX)) {
 		guint8 *appointment_body_str = (guint8 *) exchange_mapi_cal_util_camel_helper (item_data->conn, item_data->fid, item_data->mid, NULL, msg_class,
@@ -131,12 +135,7 @@ fetch_props_to_mail_item_cb (FetchItemsCallbackData *item_data, gpointer data)
 	}
 
 	if (delivery_date) {
-		NTTIME ntdate = { 0 };
-
-		ntdate = delivery_date->dwHighDateTime;
-		ntdate = ntdate << 32;
-		ntdate |= delivery_date->dwLowDateTime;
-		item->header.recieved_time = nt_time_to_unix(ntdate);
+		item->header.recieved_time = exchange_mapi_util_filetime_to_time_t (delivery_date);
 	}
 
 	if (flags && (*flags & MSGFLAG_READ) != 0)
@@ -300,7 +299,8 @@ mapi_mime_set_recipient_list (CamelMimeMessage *msg, MailItem *item)
 	GSList *l = NULL;
 	CamelInternetAddress *to_addr, *cc_addr, *bcc_addr;
 
-	g_return_if_fail (item->recipients != NULL);
+	if (!item->recipients || item->header.transport_headers)
+		return;
 
 	to_addr = camel_internet_address_new ();
 	cc_addr = camel_internet_address_new ();
@@ -356,7 +356,9 @@ mapi_mime_set_recipient_list (CamelMimeMessage *msg, MailItem *item)
 		camel_mime_message_set_recipients(msg, "Bcc", bcc_addr);
 	}
 
-	/*TODO : Unref *_addr ? */
+	g_object_unref (to_addr);
+	g_object_unref (cc_addr);
+	g_object_unref (bcc_addr);
 }
 
 static void
@@ -400,6 +402,10 @@ mapi_mime_set_msg_headers (ExchangeMapiConnection *conn, CamelMimeMessage *msg,
 
 		g_object_unref (parser);
 		g_object_unref (part);
+	} else {
+		recieved_time = item->header.recieved_time;
+		actual_time = camel_header_decode_date (ctime(&recieved_time), &offset);
+		camel_mime_message_set_date (msg, actual_time, offset);
 	}
 
 	/* Overwrite headers if we have specific properties available*/
@@ -407,11 +413,6 @@ mapi_mime_set_msg_headers (ExchangeMapiConnection *conn, CamelMimeMessage *msg,
 	if (temp_str)
 		camel_mime_message_set_subject (msg, temp_str);
 
-	recieved_time = item->header.recieved_time;
-
-	actual_time = camel_header_decode_date (ctime(&recieved_time), &offset);
-	/* camel_mime_message_set_date (msg, actual_time, offset); */
-
 	if (item->header.from) {
 		if ((item->header.from_type != NULL) && !g_utf8_collate (item->header.from_type, "EX")) {
 			gchar *from_email;
@@ -655,8 +656,11 @@ is_apple_attachment (ExchangeMAPIAttachment *attach, guint32 *data_len, guint32
 /*Takes raw attachment streams and converts to MIME Parts. Parts are added to
   either inline / non-inline lists.*/
 static void
-mapi_mime_classify_attachments (ExchangeMapiConnection *conn, mapi_id_t fid, GSList *attachments, GSList **inline_attachs, GSList **noninline)
+mapi_mime_classify_attachments (ExchangeMapiConnection *conn, mapi_id_t fid, const gchar *msg_class, GSList *attachments, GSList **inline_attachs, GSList **noninline)
 {
+	/* SMIME encrypted are without ending dot */
+	gboolean is_smime = msg_class && strstr (msg_class, ".SMIME.") > msg_class;
+
 	for (;attachments != NULL; attachments = attachments->next) {
 		ExchangeMAPIAttachment *attach = (ExchangeMAPIAttachment *)attachments->data;
 		ExchangeMAPIStream *stream = NULL;
@@ -771,7 +775,35 @@ mapi_mime_classify_attachments (ExchangeMapiConnection *conn, mapi_id_t fid, GSL
 			part = camel_mime_part_new ();
 			camel_medium_set_content (CAMEL_MEDIUM (part), CAMEL_DATA_WRAPPER (mp));
 			g_object_unref (mp);
-		} else {
+		} else if (is_smime) {
+			CamelMimeParser *parser;
+			CamelStream *mem;
+
+			mem = camel_stream_mem_new ();
+			camel_stream_write (mem, (const gchar *) stream->value->data, stream->value->len, NULL);
+			camel_stream_reset (mem, NULL);
+
+			parser = camel_mime_parser_new ();
+			camel_mime_parser_scan_from (parser, FALSE);
+			camel_mime_parser_scan_pre_from (parser, FALSE);
+			camel_mime_parser_init_with_stream (parser, mem, NULL);
+
+			if (camel_mime_parser_step (parser, NULL, NULL) == CAMEL_MIME_PARSER_STATE_HEADER
+			    && camel_mime_parser_content_type (parser) != NULL) {
+				g_object_unref (part);
+				part = camel_mime_part_new ();
+
+				camel_data_wrapper_set_mime_type_field (CAMEL_DATA_WRAPPER (part), camel_mime_parser_content_type (parser));
+				camel_mime_part_construct_content_from_parser (part, parser, NULL);
+			} else {
+				is_smime = FALSE;
+			}
+
+			g_object_unref (parser);
+			g_object_unref (mem);
+		} 
+
+		if (!is_smime && !is_apple) {
 			camel_mime_part_set_content (part, (const gchar *) stream->value->data, stream->value->len, mime_type);
 
 			content_type = camel_mime_part_get_content_type (part);
@@ -785,7 +817,7 @@ mapi_mime_classify_attachments (ExchangeMapiConnection *conn, mapi_id_t fid, GSL
 		content_id = (const gchar *) exchange_mapi_util_find_SPropVal_array_propval(attach->lpProps,
 											   PR_ATTACH_CONTENT_ID);
 		/* TODO : Add disposition */
-		if (content_id && !is_apple) {
+		if (content_id && !is_apple && !is_smime) {
 			camel_mime_part_set_content_id (part, content_id);
 			*inline_attachs = g_slist_append (*inline_attachs, part);
 		} else
@@ -805,6 +837,7 @@ mapi_mail_item_to_mime_message (ExchangeMapiConnection *conn, MailItem *item)
 
 	gboolean build_alternative = FALSE;
 	gboolean build_related = FALSE;
+	gboolean skip_set_content = FALSE;
 
 	g_return_val_if_fail (conn != NULL, NULL);
 	g_return_val_if_fail (item != NULL, NULL);
@@ -814,7 +847,7 @@ mapi_mail_item_to_mime_message (ExchangeMapiConnection *conn, MailItem *item)
 
 	mapi_mime_set_recipient_list (msg, item);
 	mapi_mime_set_msg_headers (conn, msg, item);
-	mapi_mime_classify_attachments (conn, item->fid, attach_list, &inline_attachs, &noninline_attachs);
+	mapi_mime_classify_attachments (conn, item->fid, item->msg_class, attach_list, &inline_attachs, &noninline_attachs);
 
 	build_alternative = g_slist_length (item->msg.body_parts) > 1;
 	build_related = !build_alternative && inline_attachs;
@@ -838,12 +871,34 @@ mapi_mail_item_to_mime_message (ExchangeMapiConnection *conn, MailItem *item)
 	}
 
 	if (noninline_attachs) { /* multipart/mixed */
-		multipart_body = mapi_mime_build_multipart_mixed (multipart_body,
-								  noninline_attachs);
+		if (build_alternative || build_related) {
+			multipart_body = mapi_mime_build_multipart_mixed (multipart_body, noninline_attachs);
+		} else if (g_slist_length (noninline_attachs) == 1 && item->msg_class && strstr (item->msg_class, ".SMIME") > item->msg_class) {
+			CamelMimePart *part = noninline_attachs->data;
+
+			skip_set_content = TRUE;
+
+			camel_medium_set_content (CAMEL_MEDIUM (msg), CAMEL_DATA_WRAPPER (part));
+
+			if (!strstr (item->msg_class, ".SMIME.")) {
+				/* encrypted */
+				camel_medium_set_content (CAMEL_MEDIUM (msg), camel_medium_get_content (CAMEL_MEDIUM (part)));
+				camel_mime_part_set_encoding (CAMEL_MIME_PART (msg), camel_mime_part_get_encoding (part));
+			} else {
+				/* signed */
+				camel_medium_set_content (CAMEL_MEDIUM (msg), CAMEL_DATA_WRAPPER (part));
+			}
+		} else {
+			mapi_mime_multipart_add_attachments (multipart_body, noninline_attachs);
+		}
 	}
 
-	camel_medium_set_content (CAMEL_MEDIUM (msg), CAMEL_DATA_WRAPPER(multipart_body));
+	if (!skip_set_content)
+		camel_medium_set_content (CAMEL_MEDIUM (msg), CAMEL_DATA_WRAPPER(multipart_body));
+
 	g_object_unref (multipart_body);
+	g_slist_free (inline_attachs);
+	g_slist_free (noninline_attachs);
 
 	return msg;
 }
